pub struct TaskContext {
pub kstack_top: VirtAddr,
pub rsp: u64,
pub fs_base: usize,
pub gs_base: usize,
pub ext_state: ExtendedState,
pub cr3: PhysAddr,
}Expand description
Saved hardware states of a task.
The context usually includes:
- Callee-saved registers
- Stack pointer register
- Thread pointer register (for thread-local storage, currently unsupported)
- FP/SIMD registers
On context switch, current task saves its context from CPU to memory, and the next task restores its context from memory to CPU.
On x86_64, callee-saved registers are saved to the kernel stack by the
PUSH instruction. So that rsp is the RSP after callee-saved
registers are pushed, and kstack_top is the top of the kernel stack
(RSP before any push).
Fields
kstack_top: VirtAddrThe kernel stack top of the task.
rsp: u64RSP after all callee-saved registers are pushed.
fs_base: usizeThread Local Storage (TLS).
gs_base: usizeuspace only.The gs_base register value.
ext_state: ExtendedStatefp-simd only.Extended states, i.e., FP/SIMD states.
cr3: PhysAddruspace only.The CR3 register value, i.e., the page table root.
Implementations
Sourceimpl TaskContext
impl TaskContext
Sourcepub fn init(&mut self, entry: usize, kstack_top: VirtAddr, tls_area: VirtAddr)
pub fn init(&mut self, entry: usize, kstack_top: VirtAddr, tls_area: VirtAddr)
Initializes the context for a new task, with the given entry point and kernel stack.
Sourcepub fn set_page_table_root(&mut self, cr3: PhysAddr)
Available on crate feature uspace only.
pub fn set_page_table_root(&mut self, cr3: PhysAddr)
uspace only.Changes the page table root in this context.
The hardware register for page table root (CR3 for x86) will be
updated to the next task’s after Self::switch_to.